import 'dart:async';
import 'dart:io';
import 'dart:convert';

import 'package:flutter/services.dart';

class CSV {
  static const MethodChannel _channel = const MethodChannel('csv_reader');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  Future<void> _futureDone;
  String url;
  String delimiter;
  int rowsCount;
  int columnCount;
  bool title;
  String shape;
  List<List<String>> data = [];

  /// Read from URLS
  CSV.from({this.url, this.delimiter = ",", this.title = true}) {
    _futureDone = _init();
  }

  _init() async {
    await HttpClient()
        .getUrl(Uri.parse(this.url))
        .then((request) async => await request.close())
        .then((response) async => response
                .transform(Utf8Decoder())
                .listen(// this must be awaited otherwise won't work
                    (data) {
              /// data is in arrays of strings
              List<String> rawData = LineSplitter().convert(data);

              /// row length is the length of the map as the map has the same number of lines in the CSV File
              this.rowsCount = rawData.length; // row length

              /// If ParseTitle is false, We skip, we split those strings by our delimiter
              for (int i = this.title ? 1 : 0; i < this.rowsCount; i++) {
                this.data.add(rawData[i].split(this.delimiter));
              }

              /// column length is the length of any inner array
              this.columnCount = this.data[0].length;

              /// Shape is Rows x Columns , If the CSV has a title, we've already skipped it so the shape has (Rows-1)x(Columns)
              /// as the title had a full row at the beginning
              this.shape = this.title
                  ? '${this.rowsCount}x${this.columnCount}: without title'
                  : '${this.rowsCount}x${this.columnCount}';
            }));
  }

  /// This ensures that we can await the instance of this Reader making it more flexible
  Future get initFinished => _futureDone;

  /// Allowing access of any cell by using X[i][k]
  operator [](int i) => this.data[i];
}
